@@ -1,4 +1,5 @@
 import time
+import logging
 from cgi import escape
 
 from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
@@ -8,7 +9,8 @@ from zope.publisher.interfaces.browser import IBrowserRequest
 from zope.component import adapts
 from zope.interface import Interface
 from zope.security.proxy import removeSecurityProxy
-from ZODB.utils import p64, u64, tid_repr
+from ZODB.utils import p64, u64, tid_repr, oid_repr
+from ZODB.Connection import Connection
 from persistent import Persistent
 from persistent.TimeStamp import TimeStamp
 import transaction
@@ -20,6 +22,9 @@ from zodbbrowser.state import ZodbObjectState
 from zodbbrowser.diff import compareDictsHTML
 
 
+log = logging.getLogger("zodbbrowser")
+
+
 class ZodbHelpView(BrowserView):
     """Zodb help view"""
 
@@ -71,6 +76,23 @@ class ZodbInfoView(BrowserView):
         return jar.isReadOnly()
 
     def __call__(self):
+        try:
+            return self.render()
+        finally:
+            if self.readonly:
+                resources = transaction.get()._resources
+                if resources:
+                    msg = ["Aborting changes made to:"]
+                    for r in resources:
+                        if isinstance(r, Connection):
+                            for o in r._registered_objects:
+                                msg.append("  oid=%s %s" % (oid_repr(o._p_oid), repr(o)))
+                        else:
+                            msg.append("  %s" % repr(r))
+                    log.warning("\n".join(msg))
+                transaction.abort()
+
+    def render(self):
         self.obj = None
 
         if 'oid' not in self.request:
