@@ -3,6 +3,7 @@ from cgi import escape
 from ZODB.utils import u64
 from persistent import Persistent
 from zope.component import adapts
+from zope.interface.declarations import ProvidesClass
 from zope.interface import implements, Interface
 from zope.security.proxy import removeSecurityProxy
 
@@ -20,12 +21,26 @@ class GenericValue(object):
     def __init__(self, context):
         self.context = context
 
+    def _repr(self):
+        # hook for subclasses
+        return repr(self.context)
+
     def render(self, tid=None, limit=200):
-        text = repr(self.context)
+        text = self._repr()
         if len(text) > limit:
             text = escape(text[:limit]) + '<span class="truncated">...</span>'
         else:
             text = escape(text)
+        if not isinstance(self.context, basestring):
+            try:
+                n = len(self.context)
+            except Exception:
+                pass
+            else:
+                if n == 1: # this is a crime against i18n, but oh well
+                    text += ' (%d item)' % n
+                else:
+                    text += ' (%d items)' % n
         return text
 
 
@@ -69,12 +84,27 @@ class DictValue(object):
     def __init__(self, context):
         self.context = context
 
-    def render(self, tid=None):
+    def render(self, tid=None, threshold=50):
         html = []
         for key, value in sorted(self.context.items()):
             html.append(IValueRenderer(key).render(tid) + ': ' +
                         IValueRenderer(value).render(tid))
-        return '{%s}' % ', '.join(html)
+        if sum(map(len, html)) < threshold:
+            return '{%s}' % ', '.join(html)
+        else:
+            prefix = '{<span class="dict">'
+            suffix = '</span>'
+            for n, item in enumerate(html):
+                if n == len(html) - 1:
+                    trailer = '}'
+                else:
+                    trailer = ','
+                if item.endswith(suffix):
+                    item = item[:-len(suffix)] + trailer + suffix
+                else:
+                    item += trailer
+                html[n] = item
+            return prefix + '<br />'.join(html) + suffix
 
 
 class PersistentValue(object):
@@ -95,5 +125,18 @@ class PersistentValue(object):
         if tid is not None:
             url += "&tid=%d" % u64(tid)
         value = GenericValue(self.context).render(tid)
-        return '<a href="%s">%s</a>' % (escape(url), value)
+        return '<a class="objlink" href="%s">%s</a>' % (escape(url), value)
+
+
+class ProvidesValue(GenericValue):
+    """zope.interface.Provides object renderer.
+
+    The __repr__ of zope.interface.Provides is decidedly unhelpful.
+    """
+    adapts(ProvidesClass)
+    implements(IValueRenderer)
+
+    def _repr(self):
+        return '<Provides: %s>' % ', '.join(i.__identifier__
+                                            for i in self.context._Provides__args[1:])
 
